describe("Realtime Graph Testing...", function() {

    it("If it has Realtime Controller/Store/Model defined", function() {
        const controller = Reports.app.getController('Reports.controller.RealtimeController');
        const model      = Reports.app.getModel('metric_model');
        const store      = Ext.getStore('metric_store');

        expect(controller).toBeDefined();
        expect(store).toBeDefined();
        expect(model).toBeDefined();
    });

    it("if it Converts data to add in store", function() {
        const metricstest   = helpers.textArrGenerator( R.inc(helpers.randomValue(20)) );
        const controller    = Reports.app.getController('Reports.controller.RealtimeController');
        const data          = helpers.generateServiceCheck(metricstest);
        const dataConverted = controller.convertReatimeToCp(data);

        expect(dataConverted).not.toBeUndefined();
        expect(dataConverted.cpdata).not.toBeUndefined();
        expect(dataConverted.cpdata).toEqual( jasmine.any(Array) );
    });

    it("if it Loads data into store", function() {
        const metricstest   = helpers.textArrGenerator( R.inc(helpers.randomValue(20)) );
        const controller    = Reports.app.getController('Reports.controller.RealtimeController');
        const store         = Ext.getStore('metric_store');
        const data          = helpers.generateServiceCheck(metricstest);
        const dataConverted = controller.convertReatimeToCp(data);
        const metrics       = controller.addMetricInStore(dataConverted);

        expect(dataConverted).toBeDefined();
    });

    it("If data loaded into metric store and has unique colors", function() {
        const metricstest   = helpers.textArrGenerator( helpers.randomValue(20) );
        const controller    = Reports.app.getController('Reports.controller.RealtimeController');
        const store         = Ext.getStore('metric_store');
        const data          = helpers.metricTimeline(metricstest);
        const dataConverted = data.map(controller.convertReatimeToCp);
        const metrics       = dataConverted.map(controller.addMetricInStore);

        const colorsLength  = new Colors().cpColorsAvailable();

        const getColors     = R.compose(testHelpers.breakArr(colorsLength), R.flatten, R.map(R.props(['color'])));
        const getUniqs      = R.map(R.uniq);
        const getLength     = R.map(R.length);

        const colors        = R.map(getColors, metrics);
        const colorsUniq    = R.map(getUniqs, colors);
        const isOK          = R.equals( R.map(getLength, colors), R.map(getLength, colorsUniq) );

        expect(isOK).toBe(true);
    });

});
