Ext.define('Ext.app.Config', {
    extend: 'Ext.Base',
    singleton: true,
    HOST: '0',
    SERVICE: '1',
    HOSTGROUP: '2',
    SERVICEGROUP: '3',
    reportOptions: undefined,
    usingFilter: false,
    CP: '1',
    CPDEFAULT: '0',
    CPLINE: '1',
    CPTABULAR: '2',
    CPGROUPBY: '4',
    CUSTOM: '99',
    COLORS: ["#917F8A","#8AB82D","#DC4B78","#329ACD","#6CEAD6","#B748FF","#ffc040","#11C812","#EE2C2C","#FFBF00","#A6C500","#8200c5","#12dbff","#FFB761","#6B8EE8","#D65CE4","#6CEAD6","#FFF184","#FF9393","#11C812","#EE2C2C","#FFBF00","#B2B2B2","#424242"],
    COLOR_OK: "#5dbc5e",
    COLOR_CRITICAL: "#e64747",
    COLOR_WARNING: "#f1cd5d",
    COLOR_UNKNOWN: "#b2b2b2",    
    COLOR_SPARKLINE: "#6eb0de",//"#c1ddf1",
    COLOR_PROGRESSBAR: "#d9eaf6",//"#c1ddf1",
    LONG_DATE_FORMAT: 'd/m/Y (H:i:s)',
    BASELINE_METHOD_DISABLED: "0",
    BASELINE_METHOD_TOP: "1",
    BASELINE_METHOD_BOTTOM: "2",
    BASELINE_METHOD_BOTH: "3",
    defaultReportOptions: {
        reportName: '',
        schedAsOk: '0',
        reportType: '0',
        formTo: '1',
        summarizePoints: 576,//288
        softStates: '0',
        allMayView: '0',
        sendByEmail: '0',
        timeCutId: '100',
        timeCutName: '24x7',
        selectedItens: [],
        groupbySelectMethod: 'avg',
        reportPeriod: 'last24hours',
        selectedItensDescription: [],
        contacts: [],
        showThresholdCp: '0',
        autoScale: '0',
        id: '',
        objectType: '',
        show95Percentil: '0',
        username: '',
        baselineTrend: '0',
        baselineTrendShowAlarms: '0',
        percentGraphs: '0',
        futureTrend: '0',
        monthsCounter: '1',
        absoluteMinMax: '0'
    },
    originalReportOptions: undefined,
    realtimeDataProvider: [],    
    realtimeMode: false,
    realtimePlaying: true,
    realtimeWaitingResponse: false,
    realtimeNumSamples: 50,
    realtimeUpdateInterval: 1500,
    getOptions: function(update) {
        update = update || false;
        var conf = this.reportOptions;

        if (!conf) {
            conf = Ext.clone(this.defaultReportOptions);
        } else if (update) {
            var reportform = Ext.getCmp("configuration_container");
            conf = reportform.getForm().getValues();
            conf.timeCutName = Ext.getCmp('timecut').getRawValue() || "24x7";
            conf.selectedItens = [];
            conf.selectedItensDescription = [];
            conf.contacts = [];

            var advOptions =  this.getAdvOptions();

            conf = Ext.apply(conf,advOptions);

            var grid = Ext.getCmp('metrics_available');
            grid.getSelectionModel().deselectDeletedRecords(grid.getStore().getRemovedRecords());
            var selectedMetrics = Ext.clone(grid.getSelectionModel().getSelection());

            conf.aggregatedByMetric = (Ext.getCmp('aggregate_by_metric').getValue() && (selectedMetrics.length > 0)) + 0;

            if (conf.aggregatedByMetric) {
                Ext.each(selectedMetrics, function(selected_item) {
                    conf.selectedItens.push(selected_item.get('value'));
                    conf.selectedItensDescription.push({value: selected_item.get('ids_metric_with_same_name')});
                });

                conf.objectType = Ext.app.Config.SERVICE;

            } else {
                Ext.getStore('selected_objects').each(function(selected_item) {
                    conf.selectedItens.push(selected_item.get('value'));
                    conf.selectedItensDescription.push({label: selected_item.get('name'), value: selected_item.get('value')});
                });
            }

            var selected_contacts = Ext.getCmp("email_container").getSelectionModel().getSelection();
            conf.sendByEmail = (selected_contacts.length > 0);


            for (var obj in selected_contacts) {
                conf.contacts.push(selected_contacts[obj].data.contact_id);
            }
        }

        if (conf.reportType == Ext.app.Config.CPGROUPBY){
            if (conf.reportPeriod == 'custom') {
                conf.summarizeParams = {period: conf.reportPeriod, method: conf.groupbySelectMethod, param1: conf.dateFrom+' '+conf.timeFrom, param2: conf.dateTo+' '+conf.timeTo};
            } else if (conf.reportPeriod == 'last') {
                conf.summarizeParams = {period: conf.reportPeriod, method: conf.groupbySelectMethod, param1: conf.timeCounter, param2: conf.timeUnit};
            } else {
                conf.summarizeParams = {period: conf.reportPeriod, method: conf.groupbySelectMethod, param1: '', param2: ''};
            }
        } else {
            delete conf.summarizeParams;
        }
        
        return Ext.clone(this.setOptions(conf));
    }, 
    setOptions: function(value) {
        if (!this.reportOptions) {
            this.reportOptions = Ext.clone(this.defaultReportOptions);
        }
        this.reportOptions = Ext.apply(this.reportOptions,value);

        return Ext.clone(this.reportOptions);
    },
    getAdvOptions: function() {
        var advoptions = {
            showThresholdCp: (Ext.getCmp('advanced_thresholds').checked) ? "1" : "0",
            autoScale: (Ext.getCmp('advanced_autoscale').checked) ? "1" : "0",
            show95Percentil: (Ext.getCmp('advanced_95pc').checked) ? "1" : "0",
            percentGraphs: (Ext.getCmp('advanced_graphicsinpc').checked) ? "1" : "0",
            baselineTrend: (Ext.getCmp('advanced_trendlines').checked) ? "1" : "0",
            baselineTrendShowAlarms: (Ext.getCmp('advanced_trendlines_showalarms').checked) ? "1" : "0",
            futureTrend: (Ext.getCmp('advanced_futuretrend').checked) ? "1" : "0",
            monthsCounter: Ext.getCmp('advanced_futuretrend_months').getValue(),
            absoluteMinMax: (Ext.getCmp('advanced_absoluteminmax').checked) ? "1" : "0"
        };
        return advoptions;        
    },
    setAdvOptions: function(config) {
        Ext.getCmp('advanced_thresholds').setChecked(config.showThresholdCp == "1" ? true : false);
        Ext.getCmp('advanced_autoscale').setChecked(config.autoScale == "1" ? true : false);
        Ext.getCmp('advanced_95pc').setChecked(config.show95Percentil == "1" ? true : false);
        Ext.getCmp('advanced_graphicsinpc').setChecked(config.percentGraphs == "1" ? true : false);
        Ext.getCmp('advanced_trendlines').setChecked(config.baselineTrend == "1" ? true : false);
        Ext.getCmp('advanced_trendlines_showalarms').setChecked(config.baselineTrendShowAlarms == "1" ? true : false);
        Ext.getCmp('advanced_futuretrend').setChecked(config.futureTrend == "1" ? true : false);
        Ext.getCmp('advanced_futuretrend_months').setValue(config.monthsCounter);
        Ext.getCmp('advanced_absoluteminmax').setChecked(config.absoluteMinMax == "1" ? true : false);
    }
});