Ext.define('Op.Reports.SerialChart',{
    extend: 'Ext.Base',
    config: {
        data: undefined,
        autoScale: false,
        thresholdsGuide: false,
        percentilGuide: false,
        futureGuide: false,
        baselineGuide: false,
        containerId: 'chart',
        type: 'line',
        colors: [],
        maximum: undefined,
        minimum: undefined,
        thresholdColors: {"OK":"#5dbc5e","WARNING":"#f1cd5d","CRITICAL":"#e64747","UNKNOWN":"#b2b2b2"}        
    },
    rawData: null,
    chart: undefined, 
    zoomInCallback: null,
    zoomOutCallback: null,
    zoomed: false,    
    constructor: function(config){        
        this.initConfig(config);
        this.initialize();
    },
    initialize: function() {
        var me = this;
        var chart = new AmCharts.AmSerialChart();
        chart.export = {
            "enabled": true,
            "libs": {
                "path": "resources/libs/amcharts/plugins/export/libs/"
            },
            "menu": []
        };
        chart.responsive = {
            "enabled": false
        };
        chart.theme = 'light';
        chart.language = Ext.locale.shortlang;
        chart.categoryField = "date";
        chart.columnSpacing = 0;

        var categoryAxis = chart.categoryAxis;
        categoryAxis.parseDates = true; 
        categoryAxis.minPeriod = "ss";
        categoryAxis.dashLength = 1;
        categoryAxis.gridAlpha = 0.15;
        categoryAxis.minorGridEnabled = true;
        categoryAxis.gridPosition = "start";        

        chartCursor = new AmCharts.ChartCursor();
        chartCursor.cursorPosition = "mouse";

        chartCursor.valueLineEnabled = true;
        chartCursor.valueLineBalloonEnabled = true;
        chartCursor.cursorAlpha = 0;
        chartCursor.valueLineAlpha = 0;
        chartCursor.bulletsEnabled = true;
        chartCursor.zoomable = false;
        chartCursor.categoryBalloonDateFormat = "MMM DD, YYYY - JJ:NN";

        chart.addChartCursor(chartCursor);            
        chart.balloon.borderThickness = 1;
        chart.balloon.shadowAlpha = 0;
        chart.balloon.showBullet = true;
        chart.balloon.cornerRadius = 0.1;
        
        chart.addListener("rendered", function () {           
            chart.chartCursor.addListener('selected',function(evt) {            
                if (me.zoomInCallback && chart.chartCursor.selectWithoutZooming) {
                    chart.chartCursor.clearSelection();                    
                    me.zoomInCallback(evt);
                }                
            });
        });

        this.chart = chart;

    },
    createLegend: function(container_id) {
        container_id = container_id || null;
        var legend = new AmCharts.AmLegend();
        legend.useGraphSettings = false;
        legend.autoMargins = false;
        legend.maxColumns = 1;
        this.chart.addLegend(legend,container_id);  
        this.chart.legend.addListener('hideItem',function(evt){
            me.legendHandler(evt,me);
        });
        this.chart.legend.addListener('showItem',function(evt){
            me.legendHandler(evt,me);
        }); 
    },
    enableZoom: function(enable,real) {
        this.chart.chartCursor.zoomable = enable;
        this.chart.chartCursor.selectWithoutZooming = real;
    },
    onZoomIn: function(callback) {
        this.zoomInCallback = callback;
    },
    onZoomOut: function(callback) {
        if (this.chart.chartCursor.selectWithoutZooming) {
            this.chart.clickZB = callback;            
        }
    },
    showZoomOut: function(show) {
        var me = this;

        //hack|gambi to show zoomOut button, because the version 3.19 or later doesn't support real zoom-in            
        if (show) {        
            me.chart.startTime = me.chart.startTime + 1;
            me.chart.showZB();
        }

        if (me.chart.chartCursor.zoomable && me.chart.chartCursor.selectWithoutZooming) {
            me.zoomed = show;
        }
    },
    renderChart: function() {
        var d = this.getData();
        
        if (d === undefined) {
            return;
        }
        
        var chart = this.chart;       
        

        var graphshidden = {};
        if (chart && chart.graphs) {
            chart.graphs.forEach(function(el){
                graphshidden[el.valueField] = el.hidden;                
            });
        }

        chart.graphs = [];
        chart.valueAxes = [];

        var units = {};
        var limit_max = d.limits.max;
        var limit_min = d.limits.min;

        var axis = [];
        var valueAxis = null;
        var axis_position = 'left';
        var axis_offset = 0;
        var lunit;
        //group units
        for(i=0;i<d.unit.length;i++){
            lunit = 'unit_'+d.unit[i];
            if (units.hasOwnProperty(lunit)) {
                units[lunit].max = Math.max(units[lunit].max,parseFloat(limit_max[i]));
                units[lunit].min = Math.min(units[lunit].min,parseFloat(limit_min[i]));
                units[lunit].index.push(i);
            } else {
                units[lunit] = {'unit': d.unit[i], 'max': parseFloat(limit_max[i]), 'min': parseFloat(limit_min[i]), index: [i]};
            }
        }

        for(lunit in units) {
            valueAxis = new AmCharts.ValueAxis();
            valueAxis.unit = units[lunit].unit;
            valueAxis.position = axis_position;
            valueAxis.offset = axis_offset;

            valueAxis.maximum = this.getMaximum();
            valueAxis.minimum = this.getMinimum();

            if (! this.getAutoScale() && (! this.getMaximum()) && (! this.getMinimum())){
                valueAxis.maximum = units[lunit].max;
                valueAxis.minimum = units[lunit].min;
            } else if (this.getAutoScale()) {
                valueAxis.maximum = undefined;
                valueAxis.minimum = undefined;
            }

            valueAxis.stackType = 'none';
            chart.addValueAxis(valueAxis);
            axis_position = (axis_position == 'right') ? 'left' : 'right';
            axis_offset += (axis_position == 'right') ? 50 : 0;

            valueAxis.metric_index = units[lunit].index;

            units[lunit].valueaxis = valueAxis;
        }

        chart.valueAxesDefault = chart.valueAxes.slice(0);

        var legend_data = [], c = 0, color, ef = ['_baseline_up','_baseline_down','_abs_max','_abs_min','_realistic','_optimistic','_pessimistic'];
        var findef = function(ep){ return function(e) { return (ep.indexOf(e) !== -1);};};

        var cp_color = new Colors();
        var m_color = cp_color.getCPColors();

        for (i = 0; i < d.metric.length; i++) {
            var graph = new AmCharts.AmGraph();
            graph.valueField = d.metric[i];
            graph.id = d.metric[i];
            graph.title = graph.valueField;
            graph.lineThickness = 1;
            graph.balloonText = "<span class='amCharts-balloon'>" + d.metric[i] + ": [[value]] " + d.unit[i] + '</span>';

            var metricAlreadyExist = Ext.getStore('metric_store').findRecord('metric', d.metric[i], 0, false, true, true);

            if (color && ef.some(findef(graph.valueField))){
                color = Op.Reports.Utils.shadeBlend(-0.4,m_color[c%cp_color.cpColorsAvailable()]);
            } else if (Ext.app.Config.realtimeMode) {
                color = metricAlreadyExist ? metricAlreadyExist.get('color') : m_color[c%cp_color.cpColorsAvailable()];
            } else {
                color = m_color[c%cp_color.cpColorsAvailable()];
            }

            if (d.metric.length >= i+2 && !ef.some(findef(d.metric[i+1]))) {
                c++;
            }

            if (d.metric[i].match(/(_realistic|_optimistic|_pessimistic)$/g)) {
                graph.bulletField = "bullet";

                if (d.metric[i].match(/_optimistic/g)) {
                    graph.bulletColor = "#00cc00";
                }
                if (d.metric[i].match(/_realistic/g)) {
                    graph.bulletColor = "#ff9933";
                }
                if (d.metric[i].match(/_pessimistic/g)) {
                    graph.bulletColor = "#ff3300";
                }
            }

            graph.color = color;
            graph.lineColor = graph.color;
            graph.legendColor = graph.color;
            graph.type = this.getType();

            if (graph.type == 'column') {
                chart.categoryAxis.equalSpacing = true;
                graph.fillAlphas = 0.8; 
            } else {
                chart.categoryAxis.equalSpacing = false;
                graph.fillAlphas = 0;
            }

            // start #6573
            var is_the_first_metric_but_isnt_metric_aggregated =
                ((i === 0) && (! d.host_name.match(/Metric Aggregated/)));

            if (is_the_first_metric_but_isnt_metric_aggregated) {
                graph.fillAlphas = 0.2;
            } // end #6573

            graph.columnWidth = 0.7;
            graph.minDistance = 0;

            for (lunit in units) {
                if (units[lunit].index.indexOf(i) !== -1) {
                    graph.valueAxis = units[lunit].valueaxis;
                    break;
                }
            }

            graph.gapPeriod = Number.POSITIVE_INFINITY;
            graph.connect = false; // set not connect if data missing
            
            chart.addGraph(graph);

            if (graphshidden.hasOwnProperty(graph.valueField) && graphshidden[graph.valueField]) {
                chart.hideGraph(graph);
            }
        }

        // Add circles on metrics for the graph of future trend
        if (this.getFutureGuide()) {
            var icon = "M9.875,0.625C4.697,0.625,0.5,4.822,0.5,10s4.197,9.375,9.375,9.375S19.25,15.178,19.25,10S15.053,0.625,9.875,0.625";
            var trendLines = [];
            
            var guidesGroupedByValue = (d.cpdata_future_limit || []).reduce(function(acc, metricData) {
                var fresh = {};
                fresh[metricData.value] = (acc[metricData.value] || []).concat(metricData);
                return Ext.apply(acc, fresh);
            }, {});

            var guideLines = Object.keys(guidesGroupedByValue).reduce(function(acc, key) {
                return acc.concat(
                    guidesGroupedByValue[key].reduce(function(guide, metricData) {
                        return guide.label
                            ? Ext.apply(guide, { label: guide.label + " / " + metricData.metric })
                            : {
                                value: metricData.value,
                                lineColor: "#000000",
                                lineThickness: 1,
                                dashLength: 5,
                                inside: true,
                                lineAlpha: 0.3,
                                position: "left",
                                label: metricData.metric,
                            };
                    }, new AmCharts.Guide())
                );
            }, []);

            if (d.cpdata_future_break !== null) {
                for (i = 0; i < d.cpdata_future_break.length; i++) {
                    var circleBreak = new AmCharts.TrendLine();
                    circleBreak.initialValue = d.cpdata_future_break[i].value;
                    circleBreak.finalValue = d.cpdata_future_break[i].value;
                    circleBreak.initialDate = new Date(d.cpdata_future_break[i].initial_date * 1000);
                    circleBreak.finalDate = new Date(d.cpdata_future_break[i].final_date * 1000);
                    circleBreak.above = true;
                    circleBreak.initialImage = {
                        svgPath: icon,
                        color: d.cpdata_future_break[i].color,
                        width: 8,
                        height: 8
                    };

                    trendLines.push(circleBreak);
                }
            }

            chart.trendLines = trendLines;
            chart.guides = guideLines;
        }
        
        chart.dataProvider = Ext.clone(d.cpdata);
        chart.validateData();
        chart.validateNow();
        this.addGuides();
        chart.write(this.getContainerId());
    },
    applyData: function(d) {
        if (d !== undefined) {
            d = this.fixData(d);
        }
        return d;
    },
    updateData: function(newData, oldData) {
        if (this.chart) {            
            this.renderChart();  
        }
    },
    updateAutoScale: function(newData,oldData) {
        if (this.chart) {  
            this.renderChart();
        }        
    },
    updateBaselineGuide: function(newData,oldData) {
        if (this.chart) {  
            this.renderChart();
        }        
    },
    updateFutureGuide: function(newData,oldData) {
        if (this.chart) {  
            this.renderChart();
        }        
    },
    updateThresholdsGuide: function(newData,oldData) {
        if (this.chart) {
            if (newData) {
                this.removeGuides();
                this.addGuides();              
            } else {
                this.removeGuides();              
            }          
        }      
    },
    updatePercentilGuide: function(newData,oldData) {
        if (this.chart && this.getThresholdsGuide()) {
            this.removeGuides();
            this.addGuides();
        }
    },
    removeGuides: function() {
        for (var a in this.chart.valueAxes) {
            //Utils.log('remove guide for ',chart.valueAxes[a].valueField);
            this.chart.valueAxes[a].guides = [];        
        }
        for (var c in this.chart.categoryAxis.guides) {
            if (this.chart.categoryAxis.guides[c].fixed === true) {
                continue;
            } else {
                this.chart.categoryAxis.guides[c] = [];
            }
        }

        this.chart.validateNow();
        //force zoom out button visibility
        if (this.zoomed) {
            this.showZoomOut(true);
        }
        this.addGuides();
    },
    getGraphByValueField: function(vf,use_hidden) {
        use_hidden = use_hidden || false;

        var graph = this.chart.getGraphById(vf);

        if (graph && (!use_hidden || graph.hidden === false)) {
            return graph;
        }

        return null;
    },
    checkGraph: function(vf) {        
        return  (this.getGraphByValueField(vf,true)) ? true : false;        
    },
    addGuides: function() {
        var d = this.getData();
        var chart = this.chart;  
        var guidesconfig = [];
        var gi,guide,attrname;
        //value guides
        for (var i in chart.valueAxes) {
            var valueAxis = chart.valueAxes[i];
            guidesconfig = [];
            for (var ii in valueAxis.metric_index) {
                var dindex = valueAxis.metric_index[ii];
                if (this.checkGraph(d.metric[dindex]))  {

                    is_metric_future = d.metric[dindex].match(/(_realistic|_optimistic|_pessimistic)$/g);
                    if (is_metric_future) {
                        continue;
                    }

                    if (this.getPercentilGuide()) {
                        guidesconfig.push({"label":"95% -> "+d.metric[dindex],"value":d.cpdata_perc95[dindex],"color": this.getThresholdColors().CRITICAL,"lineColor": this.getThresholdColors().CRITICAL,"dashLength": 3, "lineThickness": 1,"above": true,"position": "left"});
                    }

                    if (this.getThresholdsGuide()) {
                        guidesconfig.push({"label":Ext.locale.translate('Warning')+' -> '+d.metric[dindex],"value":d.cpdata_warn[dindex], "color": this.getThresholdColors().WARNING, "lineColor": this.getThresholdColors().WARNING, "lineThickness": 1,"above": true,"position": "left"});
                        guidesconfig.push({"label":Ext.locale.translate('Critical')+' -> '+d.metric[dindex],"value":d.cpdata_critical[dindex],"color": this.getThresholdColors().CRITICAL, "lineColor": this.getThresholdColors().CRITICAL, "lineThickness": 1,"above": true,"position": "right"});
                    }
                }
            }

            //draw value guides
            for (gi in guidesconfig) {
                guide = new AmCharts.Guide();
                guide.dashLength = 3;
                guide.inside = true;
                guide.above = true;
                guide.lineAlpha = 1;
                //Utils.assign(guidesconfig[gi],guide);

                for (attrname in guidesconfig[gi]) { 
                    guide[attrname] = guidesconfig[gi][attrname]; 
                }  
                valueAxis.addGuide(guide);
            }

        }   

        //category guides       
        var oguide = {};
        guidesconfig = [];

        //future trend guide
        if (this.getFutureGuide()) {

            var hasFixed = false;
            for (var c in chart.categoryAxis.guides) {
                if (chart.categoryAxis.guides[c].fixed === true) {
                    hasFixed = true;
                }
            }
            if (hasFixed === false) { 
                oguide = {
                    date: false,
                    toDate: false,
                    fixed: true,
                    type: 'trend',
                    fillColor: '#FAFC88',
                    fillAlpha: 0.2,
                    lineAlpha: 0
                };            

                for (var s in d.cpdata){
                    var v = d.cpdata[s];
                    if (v.isTrend === true){
                        if (!oguide.date){
                            oguide.date = v.date;
                        }
                        oguide.toDate = v.date;
                    }
                }
                guidesconfig.push(Ext.clone(oguide));
            }
        }

        //baseline guides from server
        if (this.getBaselineGuide()) {
            for (var pos in d.metric) {
                var metric = d.metric[pos];                
                if (this.checkGraph(d.metric[pos])) {                                             
                    oguide = {date: false, toDate: false, type: 'baseline', fillColor: null, balloonColor: null, balloonText:null, fillAlpha: 0.2, lineAlpha: 0};
                    if (metric.indexOf('_baseline') == -1 && d.cpdata_guides && d.cpdata_guides[pos] && d.cpdata_guides[pos].hasOwnProperty('baseline')){
                        for (var j in d.cpdata_guides[pos].baseline){
                            var vv = d.cpdata_guides[pos].baseline[j];
                            var guidetype = (vv.type == Ext.app.Config.BASELINE_METHOD_TOP) ? '_baseline_up' : '_baseline_down';
                            var baselineTrendMethodLabel = metric + guidetype; //up and down
                            //var baselineTrendMethodLabelDetails = vv.metric + ' value: ' + vv.value + ' threshold: ' + vv.threshold;
                            oguide.date = new Date(vv.start * 1000);
                            oguide.toDate = new Date(vv.end * 1000);
                            oguide.fillColor = chart.graphs[pos].color;
                            oguide.balloonColor = oguide.fillColor; 
                            oguide.balloonText = metric + ' ' +((baselineTrendMethodLabel.indexOf('_baseline_down') != -1) ?
                                                                Ext.locale.translate('Exceeded the Upper limit') : Ext.locale.translate('Exceeded the Upper limit'));                                                       
                            guidesconfig.push(Ext.clone(oguide));    
                        }
                    }                    
                }
            }
        }        

        //draw value guides
        for (gi in guidesconfig) {
            guide = new AmCharts.Guide();
            guide.inside = true;
            guide.above = true;
            for (attrname in guidesconfig[gi]) { 
                guide[attrname] = guidesconfig[gi][attrname]; 
            }  
            chart.categoryAxis.addGuide(guide);
        }

        chart.validateNow(); 

        //force zoom out button visibility
        if (this.zoomed) {
            this.showZoomOut(true);
        }
    },
    legendHandler: function(evt,me) {
        for (var i in evt.chart.graphs) {
            if (evt.dataItem.valueField == evt.chart.graphs[i].valueField) {
                evt.chart.graphs[i].hidden = (evt.type == 'hideItem') ? true : false;
            }
        }      
        me.removeGuides();
        me.addGuides();
    },
    toggleLegend: function(state) {
        state = state || false;
        var chart = this.chart;
        for (var i in chart.graphs) {
            chart[state?'showGraph':'hideGraph'](chart.graphs[i]);
        }
        this.removeGuides();
        this.addGuides();
    },    
    fixData: function (d) {
        var dp = [];
        for (i=0,len=d.cpdata.length; i < len; i++) {
            var item = d.cpdata[i];
            if (item !== null) {
                item.date = new Date(item.date * 1000); 
                dp.push(item);
            }
        }
        d.cpdata = dp;
        return d;
    }    
});

(function(AmCharts){
    AmCharts.getChartById = function (id) {
        var allCharts = AmCharts.charts;
        for (var i = 0; i < allCharts.length; i++) {
            if (id == allCharts[i].div.id) {
                return allCharts[i];
            }
        }
    };  
})(AmCharts);
